#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include "asm/fam3rtos/fam3rtos_sysctl.h"

char* onoff[] = {"off", "on"};
char* ledname[] = {"run", "alm", "err"};
char* command[] = {
   "run_off",
   "run_on",
   "alm_off",
   "alm_on",
   "err_off",
   "err_on",
   "status",
};

void help(char* s)
{
   int i;
   const int ncommand = sizeof(command)/sizeof(command[0])-1;
   fprintf(stderr, "usage : %s commands\n", s);
   fprintf(stderr, "  commands: %s\n", command[ncommand]);
   for (i=0; i<ncommand-1; i+=2) {
      fprintf(stderr, "  commands: %s, %s\n", command[i], command[i+1]);
   }
   exit(EXIT_SUCCESS);
}

int main(int argc, char* argv[])
{
   int iarg, iled;
   const int nled  = sizeof(ledname)/sizeof(ledname[0]);

   char led[4], status[4];

   if (argc<=1) {
      help(argv[0]);
   }

   const char* devicefile = "/dev/m3sysctl";

   const int fd=open(devicefile, O_RDWR);
   if (fd<0) {
      fprintf(stderr, "%s: %s\n", strerror(errno), devicefile);
      return EXIT_FAILURE;
   }

   unsigned long ledval = 0;
   for (iarg=1; iarg<argc; iarg++) {
      if (strcmp(argv[iarg], command[6])==0) {
         if (ioctl(fd, RP6X_SYSIOC_GETLED, &ledval)==EFAULT) {
            fprintf(stderr, "%s: %s\n", strerror(errno), devicefile);
            close(fd);
            return EXIT_FAILURE;
         }
         for (iled=0; iled<nled; iled++) {
            printf("%s: %s\n", ledname[iled], onoff[(ledval>>iled)&1]);
         }
      } else {
         if (sscanf(argv[iarg], "%3s_%s", led, status)!=2) {
            help(argv[0]);
         }
         if (strcmp(status, onoff[0])!=0 && strcmp(status, onoff[1])!=0) {
            help(argv[0]);
         }

         switch(led[0]) {
         case 'r': // RUN
            if (status[1]=='n') {
               ledval |= RP6X_LED_RUN_ON;
            } else {
               ledval |= RP6X_LED_RUN_OFF;
            }
            break;
         case 'a': // ALM
            if (status[1]=='n') {
               ledval |= RP6X_LED_ALM_ON;
            } else {
               ledval |= RP6X_LED_ALM_OFF;
            }
            break;
         case 'e': // ERR
            if (status[1]=='n') {
               ledval |= RP6X_LED_ERR_ON;
            } else {
               ledval |= RP6X_LED_ERR_OFF;
            }
         default:
            break;
         }
         if (ioctl(fd, RP6X_SYSIOC_SETLED, &ledval)==EFAULT) {
            fprintf(stderr, "%s: %s\n", strerror(errno), devicefile);
            close(fd);
            return EXIT_FAILURE;
         }
      }
   }

   close(fd);
   return ledval;
}
